@@ -23,7 +23,7 @@
  * @package jk_connect
  * @author      Gal Shachor <shachor@il.ibm.com>
  * @author      Mladen Turk <mturk@apache.org>
- * @version     $Revision: 603677 $
+ * @version     $Revision: 745136 $
  */
 
 
@@ -172,8 +172,9 @@ static int nb_connect(jk_sock_t sd, struct sockaddr *addr, int timeout, jk_logge
         FD_ZERO(&efdset);
         FD_SET(sd, &efdset);
 
-        tv.tv_sec  = timeout;
-        tv.tv_usec = 0;
+        tv.tv_sec = timeout / 1000;
+        tv.tv_usec = (timeout % 1000) * 1000;
+
         rc = select((int)sd + 1, NULL, &wfdset, &efdset, &tv);
         if (JK_IS_SOCKET_ERROR(rc) || rc == 0) {
             rc = WSAGetLastError();
@@ -235,8 +236,8 @@ static int nb_connect(jk_sock_t sd, struct sockaddr *addr, int timeout, jk_logge
 
         FD_ZERO(&wfdset);
         FD_SET(sd, &wfdset);
-        tv.tv_sec  = timeout;
-        tv.tv_usec = 0;
+        tv.tv_sec = timeout / 1000;
+        tv.tv_usec = (timeout % 1000) * 1000;
         rc = select(sd + 1, NULL, &wfdset, NULL, &tv);
         if (rc <= 0) {
             /* Save errno */
@@ -315,7 +316,8 @@ in_addr_t jk_inet_addr(const char * addrstr)
  * @return         JK_FALSE: some kind of error occured
  *                 JK_TRUE: success
  */
-int jk_resolve(const char *host, int port, struct sockaddr_in *rc, jk_logger_t *l)
+int jk_resolve(const char *host, int port, struct sockaddr_in *rc,
+               void *pool, jk_logger_t *l)
 {
     int x;
     struct in_addr laddr;
@@ -342,11 +344,12 @@ int jk_resolve(const char *host, int port, struct sockaddr_in *rc, jk_logger_t *
         char *remote_ipaddr;
 
         if (!jk_apr_pool) {
-            if (apr_pool_create(&jk_apr_pool, NULL) != APR_SUCCESS) {
+            if (apr_pool_create(&jk_apr_pool, (apr_pool_t *)pool) != APR_SUCCESS) {
                 JK_TRACE_EXIT(l);
                 return JK_FALSE;
             }
         }
+        apr_pool_clear(jk_apr_pool);
         if (apr_sockaddr_info_get
             (&remote_sa, host, APR_UNSPEC, (apr_port_t) port, 0, jk_apr_pool)
             != APR_SUCCESS) {
@@ -413,7 +416,8 @@ int jk_resolve(const char *host, int port, struct sockaddr_in *rc, jk_logger_t *
  * @remark          Cares about errno
  */
 jk_sock_t jk_open_socket(struct sockaddr_in *addr, int keepalive,
-                         int timeout, int sock_buf, jk_logger_t *l)
+                         int timeout, int connect_timeout,
+                         int sock_buf, jk_logger_t *l)
 {
     char buf[32];
     jk_sock_t sd;
@@ -448,6 +452,29 @@ jk_sock_t jk_open_socket(struct sockaddr_in *addr, int keepalive,
         jk_log(l, JK_LOG_DEBUG,
                "socket TCP_NODELAY set to On");
     if (keepalive) {
+#if defined(WIN32) && !defined(NETWARE)
+        DWORD dw;
+        struct tcp_keepalive ka = { 0 }, ks = { 0 };
+        if (timeout)
+            ka.keepalivetime = timeout * 10000;
+        else
+            ka.keepalivetime = 60 * 10000; /* 10 minutes */
+        ka.keepaliveinterval = 1000;
+        ka.onoff = 1;
+        if (WSAIoctl(sd, SIO_KEEPALIVE_VALS, &ka, sizeof(ka),
+                     &ks, sizeof(ks), &dw, NULL, NULL)) {
+            JK_GET_SOCKET_ERRNO();
+            jk_log(l, JK_LOG_ERROR,
+                   "failed setting SIO_KEEPALIVE_VALS (errno=%d)", errno);
+            jk_close_socket(sd, l);
+            JK_TRACE_EXIT(l);
+            return JK_INVALID_SOCKET;                                                
+        }
+        if (JK_IS_DEBUG_LEVEL(l))
+            jk_log(l, JK_LOG_DEBUG,
+                   "socket SO_KEEPALIVE set to %d seconds",
+                   ka.keepalivetime / 1000);
+#else
         set = 1;
         if (setsockopt(sd, SOL_SOCKET, SO_KEEPALIVE, (SET_TYPE)&set,
                        sizeof(set))) {
@@ -461,6 +488,7 @@ jk_sock_t jk_open_socket(struct sockaddr_in *addr, int keepalive,
         if (JK_IS_DEBUG_LEVEL(l))
             jk_log(l, JK_LOG_DEBUG,
                    "socket SO_KEEPALIVE set to On");
+#endif
     }
 
     if (sock_buf > 0) {
@@ -554,7 +582,7 @@ iSeries when Unix98 is required at compil time */
 #if (_XOPEN_SOURCE >= 520) && defined(AS400)
     ((struct sockaddr *)addr)->sa_len = sizeof(struct sockaddr_in);
 #endif
-    ret = nb_connect(sd, (struct sockaddr *)addr, timeout, l);
+    ret = nb_connect(sd, (struct sockaddr *)addr, connect_timeout, l);
 #if defined(WIN32) || (defined(NETWARE) && defined(__NOVELL_LIBC__))
     if (JK_IS_SOCKET_ERROR(ret)) {
         JK_GET_SOCKET_ERRNO();
@@ -635,6 +663,7 @@ int jk_shutdown_socket(jk_sock_t sd, jk_logger_t *l)
 {
     char dummy[512];
     int rc = 0;
+    int rd = 0;
     int save_errno;
     fd_set rs;
     struct timeval tv;
@@ -648,11 +677,16 @@ int jk_shutdown_socket(jk_sock_t sd, jk_logger_t *l)
     }
 
     save_errno = errno;
+    if (JK_IS_DEBUG_LEVEL(l))
+        jk_log(l, JK_LOG_DEBUG, "About to shutdown socket %d", sd);
+
     /* Shut down the socket for write, which will send a FIN
      * to the peer.
      */
     if (shutdown(sd, SHUT_WR)) {
         rc = jk_close_socket(sd, l);
+        if (JK_IS_DEBUG_LEVEL(l))
+            jk_log(l, JK_LOG_DEBUG, "Failed sending SHUT_WR for socket %d", sd);
         errno = save_errno;
         JK_TRACE_EXIT(l);
         return rc;
@@ -681,6 +715,8 @@ int jk_shutdown_socket(jk_sock_t sd, jk_logger_t *l)
 #else
                 rc = read(sd, &dummy[0], sizeof(dummy));
 #endif
+                if (rc > 0)
+                    rd += rc;
             } while (JK_IS_SOCKET_ERROR(rc) && (errno == EINTR || errno == EAGAIN));
 
             if (rc <= 0)
@@ -692,6 +728,9 @@ int jk_shutdown_socket(jk_sock_t sd, jk_logger_t *l)
     } while (difftime(time(NULL), start) < MAX_SECS_TO_LINGER);
 
     rc = jk_close_socket(sd, l);
+    if (JK_IS_DEBUG_LEVEL(l))
+        jk_log(l, JK_LOG_DEBUG, "Shutdown socket %d and read %d lingering bytes",
+               sd, rd);
     errno = save_errno;
     JK_TRACE_EXIT(l);
     return rc;
@@ -778,12 +817,13 @@ int jk_tcp_socket_recvfull(jk_sock_t sd, unsigned char *b, int len, jk_logger_t
 #else
             rd = read(sd, (char *)b + rdlen, len - rdlen);
 #endif
-        } while (JK_IS_SOCKET_ERROR(rd) && (errno == EINTR || errno == EAGAIN));
+        } while (JK_IS_SOCKET_ERROR(rd) && errno == EINTR);
 
         if (JK_IS_SOCKET_ERROR(rd)) {
             jk_shutdown_socket(sd, l);
             JK_TRACE_EXIT(l);
-            return (errno > 0) ? -errno : errno;
+            rd = (errno > 0) ? -errno : errno;
+            return (rd == 0) ? JK_SOCKET_EOF : rd;
         }
         else if (rd == 0) {
             jk_shutdown_socket(sd, l);
