@@ -18,7 +18,7 @@
 /***************************************************************************
  * Description: Logger object definitions                                  *
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
- * Version:     $Revision: 522764 $                                           *
+ * Version:     $Revision: 596813 $                                           *
  ***************************************************************************/
 
 #ifndef JK_LOGGER_H
@@ -31,14 +31,21 @@ extern "C"
 {
 #endif
 
+#define JK_TIME_MAX_SIZE (64)
+
 typedef struct jk_logger jk_logger_t;
 struct jk_logger
 {
     void *logger_private;
     int level;
-    const char *log_fmt;
+    const char *log_fmt;                   /* the configured timestamp format for logging */
+    char log_fmt_subsec[JK_TIME_MAX_SIZE]; /* like log_fmt, but milli/micro seconds marker
+                                              replaced, because strftime() doesn't handle those */
+    int    log_fmt_type;                   /* do we want milli or microseconds */
+    size_t log_fmt_offset;                 /* at which position should we insert */
+    size_t log_fmt_size;                   /* how long is this format string */
 
-    int (JK_METHOD * log) (jk_logger_t *l, int level, const char *what);
+    int (JK_METHOD * log) (jk_logger_t *l, int level, int used, char *what);
 
 };
 
