@@ -20,7 +20,7 @@
  * Author:      Costin <costin@costin.dnt.ro>                              *
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
  * Author:      Henri Gomez <hgomez@apache.org>                            *
- * Version:     $Revision: 551809 $                                          *
+ * Version:     $Revision: 705882 $                                          *
  ***************************************************************************/
 
 #include "jk_pool.h"
@@ -40,6 +40,10 @@ void jk_b_reset(jk_msg_buf_t *msg)
 {
     msg->len = 4;
     msg->pos = 4;
+    if (msg->buf && msg->maxlen) {
+        /* Clear the message buffer */
+        memset(msg->buf, 0, msg->maxlen);
+    }
 }
 
 int jk_b_append_long(jk_msg_buf_t *msg, unsigned long val)
@@ -105,7 +109,7 @@ jk_msg_buf_t *jk_b_new(jk_pool_t *p)
     if (!msg) {
         return NULL;
     }
-
+    memset(msg, 0, sizeof(jk_msg_buf_t));
     msg->pool = p;
 
     return msg;
@@ -275,8 +279,10 @@ unsigned char *jk_b_get_string(jk_msg_buf_t *msg)
     int start = msg->pos;
 
     if ((size == 0xFFFF) || (size + start > msg->maxlen)) {
-        /* TODO: return NULL and deal with that in the code */
-        return (unsigned char *)"ERROR";
+        /* Error of overflow in AJP packet.
+         * The complete message is probably invalid.
+         */
+        return NULL;
     }
 
     msg->pos += size;
