@@ -22,7 +22,7 @@
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
  * Author:      Dan Milstein <danmil@shore.net>                            *
  * Author:      Henri Gomez <hgomez@apache.org>                            *
- * Version:     $Revision: 604384 $                                          *
+ * Version:     $Revision: 745842 $                                          *
  ***************************************************************************/
 
 #ifndef JK_SERVICE_H
@@ -51,6 +51,9 @@ extern "C"
 struct jk_worker_env
 {
 
+    /* The original configuration map */
+    jk_map_t *init_data;
+
     /* The URI to WORKER map, will be feeded by AJP14 autoconf feature */
     jk_uri_worker_map_t *uri_to_worker;
 
@@ -62,6 +65,9 @@ struct jk_worker_env
 
     /* Virtual server handled - "*" is all virtual */
     char *virtual;
+    
+    /* Optional APR pool used for configuration */
+    void *pool;
 };
 typedef struct jk_worker_env jk_worker_env_t;
 
@@ -72,6 +78,23 @@ typedef struct jk_ws_service jk_ws_service_t;
 typedef struct jk_endpoint jk_endpoint_t;
 typedef struct jk_worker jk_worker_t;
 
+struct svc_extension
+{
+    /* reply_timeout overwrite */
+    int reply_timeout;
+    /* activation state overwrites for load balancers */
+    /* Dynamically allocated array with one entry per lb member. */
+    int *activation;
+    /* fail_on_status overwrites */
+    /* Number of elements in the array fail_on_status. */
+    int fail_on_status_size;
+    /* Dynamically allocated array with one entry per status. */
+    int *fail_on_status;
+    /* Use server error pages for responses >= 400. */
+    int use_server_error_pages;
+};
+typedef struct svc_extension svc_extension_t;
+
 /*
  * The web server service 'class'.  An instance of this class is created
  * for each request which is forwarded from the web server to the servlet
@@ -208,10 +231,6 @@ struct jk_ws_service
     jk_msg_buf_t *reco_buf;
     int reco_status;
 
-    /* Number of retries. Defaults to JK_RETRIES
-     */
-    int retries;
-
     /*
      * If set call flush after each write
      */
@@ -222,11 +241,21 @@ struct jk_ws_service
      */
     int flush_header;
 
+    /*
+     * service extensions
+     */
+    svc_extension_t extension;
+
     /*
      * JK_TRUE if response headers have been sent back
      */
     int response_started;
 
+    /*
+     * JK_TRUE if response should not be send to the client
+     */
+    int response_blocked;
+
     /*
      * HTTP status sent from container.
      */
@@ -271,6 +300,11 @@ struct jk_ws_service
      */
     void (JK_METHOD * flush) (jk_ws_service_t *s);
 
+    /*
+     * Done with sending response back to the browser.
+     */
+    void (JK_METHOD * done) (jk_ws_service_t *s);
+
     /*
      * If set do not reuse socket after each full response
      */
@@ -415,11 +449,8 @@ struct jk_endpoint
 struct jk_worker
 {
 
-    /*
-     * Public property to enable the number of retry attempts
-     * on this worker.
-     */
-    int retries;
+    jk_worker_env_t *we;
+
     /*
      * A 'this' pointer which is used by the subclasses of this class to
      * point to data/functions which are specific to a given protocol
