@@ -19,13 +19,16 @@
  * Description: Shared Memory support                                      *
  * Author:      Mladen Turk <mturk@jboss.com>                              *
  * Author:      Rainer Jung <rjung@apache.org>                             *
- * Version:     $Revision: 551893 $                                        *
+ * Version:     $Revision: 704548 $                                        *
  ***************************************************************************/
 
 #include "jk_global.h"
 #include "jk_pool.h"
 #include "jk_util.h"
 #include "jk_mt.h"
+#include "jk_lb_worker.h"
+#include "jk_ajp13_worker.h"
+#include "jk_ajp14_worker.h"
 #include "jk_shm.h"
 
 /** jk shm header core data structure */
@@ -58,6 +61,9 @@ typedef struct jk_shm_header jk_shm_header_t;
 struct jk_shm
 {
     size_t     size;
+    unsigned   ajp_workers;
+    unsigned   lb_sub_workers;
+    unsigned   lb_workers;
     char       *filename;
     char       *lockname;
     int        fd;
@@ -70,13 +76,74 @@ struct jk_shm
 typedef struct jk_shm jk_shm_t;
 
 static const char shm_signature[] = { JK_SHM_MAGIC };
-static jk_shm_t jk_shmem = { 0, NULL, NULL, -1, -1, 0, NULL};
+static jk_shm_t jk_shmem = { 0, 0, 0, 0, NULL, NULL, -1, -1, 0, NULL};
 static time_t jk_workers_modified_time = 0;
 static time_t jk_workers_access_time = 0;
 #if defined (WIN32)
 static HANDLE jk_shm_map = NULL;
+static HANDLE jk_shm_hlock = NULL;
 #endif
 
+/* Calculate needed shm size */
+size_t jk_shm_calculate_size(jk_map_t *init_data, jk_logger_t *l)
+{
+    char **worker_list;
+    unsigned i;
+    unsigned num_of_workers;
+    int num_of_ajp_workers = 0;
+    int num_of_lb_sub_workers = 0;
+    int num_of_lb_workers = 0;
+
+    JK_TRACE_ENTER(l);
+
+    if (jk_get_worker_list(init_data, &worker_list,
+                           &num_of_workers) == JK_FALSE) {
+        jk_log(l, JK_LOG_ERROR,
+               "Could not get worker list from map");
+        JK_TRACE_EXIT(l);
+        return 0;
+    }
+
+    for (i = 0; i < num_of_workers; i++) {
+        const char *type = jk_get_worker_type(init_data, worker_list[i]);
+
+        if (!strcmp(type, JK_AJP13_WORKER_NAME) ||
+            !strcmp(type, JK_AJP14_WORKER_NAME)) {
+            num_of_ajp_workers++;
+        }
+        else if (!strcmp(type, JK_LB_WORKER_NAME)) {
+            char **member_list;
+            unsigned num_of_members;
+            num_of_lb_workers++;
+            if (jk_get_lb_worker_list(init_data, worker_list[i],
+                                      &member_list, &num_of_members) == JK_FALSE) {
+                jk_log(l, JK_LOG_ERROR,
+                       "Could not get member list for lb worker from map");
+            }
+            else {
+                if (JK_IS_DEBUG_LEVEL(l))
+                    jk_log(l, JK_LOG_DEBUG, "worker %s of type %s has %u members",
+                           worker_list[i], JK_LB_WORKER_NAME, num_of_members);
+                num_of_lb_sub_workers += num_of_members;
+            }
+        }
+    }
+    if (JK_IS_DEBUG_LEVEL(l))
+        jk_log(l, JK_LOG_DEBUG, "shared memory will contain %d ajp workers of size %d and %d lb workers of size %d with %d members of size %d+%d",
+               num_of_ajp_workers, JK_SHM_AJP_SIZE(1),
+               num_of_lb_workers, JK_SHM_LB_SIZE(1),
+               num_of_lb_sub_workers, JK_SHM_LB_SUB_SIZE(1), JK_SHM_AJP_SIZE(1));
+    jk_shmem.ajp_workers = num_of_ajp_workers;
+    jk_shmem.lb_sub_workers = num_of_lb_sub_workers;
+    jk_shmem.lb_workers = num_of_lb_workers;
+    JK_TRACE_EXIT(l);
+    return JK_SHM_AJP_SIZE(jk_shmem.ajp_workers) +
+           JK_SHM_LB_SUB_SIZE(jk_shmem.lb_sub_workers) +
+           JK_SHM_AJP_SIZE(jk_shmem.lb_sub_workers) +
+           JK_SHM_LB_SIZE(jk_shmem.lb_workers);
+}
+
+
 #if defined (WIN32) || defined(NETWARE)
 
 /* Use plain memory */
@@ -84,6 +151,7 @@ int jk_shm_open(const char *fname, size_t sz, jk_logger_t *l)
 {
     int rc;
     int attached = 0;
+    char lkname[MAX_PATH];
     JK_TRACE_ENTER(l);
     if (jk_shmem.hdr) {
         if (JK_IS_DEBUG_LEVEL(l))
@@ -92,7 +160,7 @@ int jk_shm_open(const char *fname, size_t sz, jk_logger_t *l)
         return 0;
     }
 
-    jk_shmem.size =  JK_SHM_ALIGN(sizeof(jk_shm_header_t) + sz);
+    jk_shmem.size = JK_SHM_ALIGN(sizeof(jk_shm_header_t) + sz);
 
 #if defined (WIN32)
     if (fname) {
@@ -112,6 +180,19 @@ int jk_shm_open(const char *fname, size_t sz, jk_logger_t *l)
             JK_TRACE_EXIT(l);
             return -1;
         }
+        sprintf(lkname, "Global\\%s_MUTEX", fname);
+        if (attached) {
+            jk_shm_hlock = OpenMutex(MUTEX_ALL_ACCESS, FALSE, lkname);
+        }
+        else {
+            jk_shm_hlock = CreateMutex(NULL, FALSE, lkname);            
+        }
+        if (jk_shm_hlock == NULL || jk_shm_hlock == INVALID_HANDLE_VALUE) {
+            CloseHandle(jk_shm_map);
+            jk_shm_map = NULL;
+            JK_TRACE_EXIT(l);
+            return -1;
+        }
         jk_shmem.hdr = (jk_shm_header_t *)MapViewOfFile(jk_shm_map,
                                                         FILE_MAP_ALL_ACCESS,
                                                         0,
@@ -127,6 +208,10 @@ int jk_shm_open(const char *fname, size_t sz, jk_logger_t *l)
             CloseHandle(jk_shm_map);
             jk_shm_map = NULL;
         }
+        if (jk_shm_hlock) {
+            CloseHandle(jk_shm_hlock);
+            jk_shm_hlock = NULL;
+        }
 #endif
         JK_TRACE_EXIT(l);
         return -1;
@@ -157,7 +242,7 @@ int jk_shm_open(const char *fname, size_t sz, jk_logger_t *l)
         if (jk_shmem.hdr->h.data.childs > 1) {
             if (JK_IS_DEBUG_LEVEL(l)) {
                 jk_log(l, JK_LOG_DEBUG,
-                       "Reseting the shared memory for child %d",
+                       "Resetting the shared memory for child %d",
                        jk_shmem.hdr->h.data.childs);
             }
         }
@@ -167,9 +252,11 @@ int jk_shm_open(const char *fname, size_t sz, jk_logger_t *l)
     JK_INIT_CS(&(jk_shmem.cs), rc);
     if (JK_IS_DEBUG_LEVEL(l))
         jk_log(l, JK_LOG_DEBUG,
-               "%s shared memory size=%u free=%u addr=%#lx",
+               "%s shared memory %s size=%u free=%u addr=%#lx",
                attached ? "Attached" : "Initialized",
-               jk_shmem.size, jk_shmem.hdr->h.data.size, jk_shmem.hdr);
+               jk_shm_name(), jk_shmem.size,
+               jk_shmem.hdr->h.data.size - jk_shmem.hdr->h.data.pos,
+               jk_shmem.hdr);
     JK_TRACE_EXIT(l);
     return 0;
 }
@@ -182,8 +269,8 @@ int jk_shm_attach(const char *fname, size_t sz, jk_logger_t *l)
             jk_shmem.attached = 1;
             if (JK_IS_DEBUG_LEVEL(l)) {
                 jk_log(l, JK_LOG_DEBUG,
-                   "Attached shared memory [%d] size=%u free=%u addr=%#lx",
-                   jk_shmem.hdr->h.data.childs, jk_shmem.hdr->h.data.size,
+                   "Attached shared memory %s [%d] size=%u free=%u addr=%#lx",
+                   jk_shm_name(), jk_shmem.hdr->h.data.childs, jk_shmem.size,
                    jk_shmem.hdr->h.data.size - jk_shmem.hdr->h.data.pos,
                    jk_shmem.hdr);
             }
@@ -202,6 +289,10 @@ void jk_shm_close()
     if (jk_shmem.hdr) {
         int rc;
 #if defined (WIN32)
+        if (jk_shm_hlock) {
+            CloseHandle(jk_shm_hlock);
+            jk_shm_hlock = NULL;
+        }
         if (jk_shm_map) {
             --jk_shmem.hdr->h.data.childs;
             UnmapViewOfFile(jk_shmem.hdr);
@@ -445,8 +536,10 @@ static int do_shm_open(const char *fname, int attached,
         jk_shmem.hdr->h.data.childs = 1;
         if (JK_IS_DEBUG_LEVEL(l))
             jk_log(l, JK_LOG_DEBUG,
-                   "Initialized shared memory size=%u free=%u addr=%#lx",
-                   jk_shmem.size, jk_shmem.hdr->h.data.size, jk_shmem.hdr);
+                   "Initialized shared memory %s size=%u free=%u addr=%#lx",
+                   jk_shm_name(), jk_shmem.size,
+                   jk_shmem.hdr->h.data.size - jk_shmem.hdr->h.data.pos,
+                   jk_shmem.hdr);
     }
     else {
         unsigned int nchild;
@@ -455,8 +548,8 @@ static int do_shm_open(const char *fname, int attached,
         nchild = jk_shmem.hdr->h.data.childs;
         if (JK_IS_DEBUG_LEVEL(l))
             jk_log(l, JK_LOG_DEBUG,
-                   "Attached shared memory [%d] size=%u free=%u addr=%#lx",
-                   nchild, jk_shmem.hdr->h.data.size,
+                   "Attached shared memory %s [%d] size=%u free=%u addr=%#lx",
+                   jk_shm_name(), nchild, jk_shmem.size,
                    jk_shmem.hdr->h.data.size - jk_shmem.hdr->h.data.pos,
                    jk_shmem.hdr);
         /*
@@ -469,7 +562,7 @@ static int do_shm_open(const char *fname, int attached,
         if (nchild > 1) {
             if (JK_IS_DEBUG_LEVEL(l)) {
                 jk_log(l, JK_LOG_DEBUG,
-                       "Reseting the shared memory for child %d",
+                       "Resetting the shared memory for child %d",
                        nchild);
             }
         }
@@ -584,7 +677,7 @@ void *jk_shm_alloc(jk_pool_t *p, size_t size)
     void *rc = NULL;
 
     if (jk_shmem.hdr) {
-        size = JK_ALIGN_DEFAULT(size);
+        size = JK_SHM_ALIGN(size);
         if ((jk_shmem.hdr->h.data.size - jk_shmem.hdr->h.data.pos) >= size) {
             rc = &(jk_shmem.hdr->buf[jk_shmem.hdr->h.data.pos]);
             jk_shmem.hdr->h.data.pos += size;
@@ -637,9 +730,19 @@ int jk_shm_lock()
 {
     int rc;
     JK_ENTER_CS(&(jk_shmem.cs), rc);
+#if defined (WIN32)
+    if (rc == JK_TRUE && jk_shm_hlock != NULL) {
+        DWORD rv = WaitForSingleObject(jk_shm_hlock, INFINITE);
+        if (rv == WAIT_OBJECT_0 || rv == WAIT_ABANDONED)
+            rc = JK_TRUE;
+        else
+            rc = JK_FALSE;         
+    }
+#else    
     if (rc == JK_TRUE && jk_shmem.fd_lock != -1) {
         JK_ENTER_LOCK(jk_shmem.fd_lock, rc);
     }
+#endif
     return rc;
 }
 
@@ -647,23 +750,63 @@ int jk_shm_unlock()
 {
     int rc;
     JK_LEAVE_CS(&(jk_shmem.cs), rc);
+#if defined (WIN32)
+    if (rc == JK_TRUE && jk_shm_hlock != NULL) {
+        if (!ReleaseMutex(jk_shm_hlock))
+            rc = JK_FALSE;         
+    }
+#else    
     if (rc == JK_TRUE && jk_shmem.fd_lock != -1) {
         JK_LEAVE_LOCK(jk_shmem.fd_lock, rc);
     }
+#endif
     return rc;
 }
 
-jk_shm_worker_t *jk_shm_alloc_worker(jk_pool_t *p)
+jk_shm_ajp_worker_t *jk_shm_alloc_ajp_worker(jk_pool_t *p)
 {
-    jk_shm_worker_t *w = (jk_shm_worker_t *)jk_shm_alloc(p, sizeof(jk_shm_worker_t));
+    jk_shm_ajp_worker_t *w = (jk_shm_ajp_worker_t *)jk_shm_alloc(p, JK_SHM_AJP_WORKER_SIZE);
     if (w) {
-        memset(w, 0, sizeof(jk_shm_worker_t));
+        memset(w, 0, JK_SHM_AJP_WORKER_SIZE);
         if (jk_shmem.hdr) {
             jk_shmem.hdr->h.data.workers++;
-            w->id = jk_shmem.hdr->h.data.workers;
+            w->h.id = jk_shmem.hdr->h.data.workers;
+            w->h.type = JK_AJP13_WORKER_TYPE;
         }
         else
-            w->id = -1;
+            w->h.id = -1;
+    }
+    return w;
+}
+
+jk_shm_lb_sub_worker_t *jk_shm_alloc_lb_sub_worker(jk_pool_t *p)
+{
+    jk_shm_lb_sub_worker_t *w = (jk_shm_lb_sub_worker_t *)jk_shm_alloc(p, JK_SHM_LB_SUB_WORKER_SIZE);
+    if (w) {
+        memset(w, 0, JK_SHM_LB_SUB_WORKER_SIZE);
+        if (jk_shmem.hdr) {
+            jk_shmem.hdr->h.data.workers++;
+            w->h.id = jk_shmem.hdr->h.data.workers;
+            w->h.type = JK_LB_SUB_WORKER_TYPE;
+        }
+        else
+            w->h.id = -1;
+    }
+    return w;
+}
+
+jk_shm_lb_worker_t *jk_shm_alloc_lb_worker(jk_pool_t *p)
+{
+    jk_shm_lb_worker_t *w = (jk_shm_lb_worker_t *)jk_shm_alloc(p, JK_SHM_LB_WORKER_SIZE);
+    if (w) {
+        memset(w, 0, JK_SHM_LB_WORKER_SIZE);
+        if (jk_shmem.hdr) {
+            jk_shmem.hdr->h.data.workers++;
+            w->h.id = jk_shmem.hdr->h.data.workers;
+            w->h.type = JK_LB_WORKER_TYPE;
+        }
+        else
+            w->h.id = -1;
     }
     return w;
 }
