@@ -20,7 +20,7 @@
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
  * Author:      Henri Gomez <hgomez@apache.org>                            *
  * Author:      Rainer Jung <rjung@apache.org>                             *
- * Version:     $Revision: 604807 $                                          *
+ * Version:     $Revision: 752016 $                                          *
  ***************************************************************************/
 
 
@@ -52,13 +52,18 @@
 #define CACHE_OF_WORKER_MIN         ("connection_pool_minsize")
 #define CACHE_TIMEOUT_DEPRECATED    ("cache_timeout")
 #define CACHE_TIMEOUT_OF_WORKER     ("connection_pool_timeout")
+#define CACHE_ACQUIRE_OF_WORKER     ("connection_acquire_timeout")
 #define RECOVERY_OPTS_OF_WORKER     ("recovery_options")
 #define CONNECT_TIMEOUT_OF_WORKER   ("connect_timeout")
 #define PREPOST_TIMEOUT_OF_WORKER   ("prepost_timeout")
 #define REPLY_TIMEOUT_OF_WORKER     ("reply_timeout")
 #define SOCKET_TIMEOUT_OF_WORKER    ("socket_timeout")
+#define SOCKET_CONNECT_TIMEOUT_OF_WORKER ("socket_connect_timeout")
+#define PING_TIMEOUT_OF_WORKER      ("ping_timeout")
+#define PING_MODE_OF_WORKER         ("ping_mode")
 #define SOCKET_BUFFER_OF_WORKER     ("socket_buffer")
 #define SOCKET_KEEPALIVE_OF_WORKER  ("socket_keepalive")
+#define CONN_PING_INTERVAL_OF_WORKER  ("connection_ping_interval")
 #define RECYCLE_TIMEOUT_DEPRECATED  ("recycle_timeout")
 #define LOAD_FACTOR_OF_WORKER       ("lbfactor")
 #define DISTANCE_OF_WORKER          ("distance")
@@ -66,6 +71,9 @@
 #define BALANCE_WORKERS             ("balance_workers")
 #define STICKY_SESSION              ("sticky_session")
 #define STICKY_SESSION_FORCE        ("sticky_session_force")
+#define SESSION_COOKIE_OF_WORKER    ("session_cookie")
+#define SESSION_PATH_OF_WORKER      ("session_path")
+
 #define LOCAL_WORKER_DEPRECATED     ("local_worker")
 #define LOCAL_WORKER_ONLY_DEPRECATED ("local_worker_only")
 #define JVM_ROUTE_OF_WORKER_DEPRECATED ("jvm_route")
@@ -79,7 +87,9 @@
 #define IS_WORKER_STOPPED_DEPRECATED  ("stopped")
 #define ACTIVATION_OF_WORKER        ("activation")
 #define WORKER_RECOVER_TIME         ("recover_time")
+#define WORKER_ERROR_ESCALATION_TIME ("error_escalation_time")
 #define MAX_REPLY_TIMEOUTS_OF_WORKER ("max_reply_timeouts")
+#define RETRY_INTERVAL_OF_WORKER    ("retry_interval")
 #define WORKER_MAX_PACKET_SIZE      ("max_packet_size")
 #define STYLE_SHEET_OF_WORKER       ("css")
 #define NAMESPACE_OF_WORKER         ("ns")
@@ -166,17 +176,24 @@ static const char *unique_properties[] = {
     CACHE_OF_WORKER_MIN,
     CACHE_TIMEOUT_DEPRECATED,
     CACHE_TIMEOUT_OF_WORKER,
+    CACHE_ACQUIRE_OF_WORKER,
     RECOVERY_OPTS_OF_WORKER,
     CONNECT_TIMEOUT_OF_WORKER,
     PREPOST_TIMEOUT_OF_WORKER,
+    PING_TIMEOUT_OF_WORKER,
+    PING_MODE_OF_WORKER,
     REPLY_TIMEOUT_OF_WORKER,
     SOCKET_TIMEOUT_OF_WORKER,
+    SOCKET_CONNECT_TIMEOUT_OF_WORKER,
     SOCKET_BUFFER_OF_WORKER,
     SOCKET_KEEPALIVE_OF_WORKER,
+    CONN_PING_INTERVAL_OF_WORKER,
     RECYCLE_TIMEOUT_DEPRECATED,
     LOAD_FACTOR_OF_WORKER,
     STICKY_SESSION,
     STICKY_SESSION_FORCE,
+    SESSION_COOKIE_OF_WORKER,
+    SESSION_PATH_OF_WORKER,
     LOCAL_WORKER_DEPRECATED,
     LOCAL_WORKER_ONLY_DEPRECATED,
     JVM_ROUTE_OF_WORKER_DEPRECATED,
@@ -189,7 +206,9 @@ static const char *unique_properties[] = {
     IS_WORKER_STOPPED_DEPRECATED,
     ACTIVATION_OF_WORKER,
     WORKER_RECOVER_TIME,
+    WORKER_ERROR_ESCALATION_TIME,
     MAX_REPLY_TIMEOUTS_OF_WORKER,
+    RETRY_INTERVAL_OF_WORKER,
     WORKER_MAX_PACKET_SIZE,
     STYLE_SHEET_OF_WORKER,
     READ_ONLY_OF_WORKER,
@@ -249,13 +268,18 @@ static const char *supported_properties[] = {
     CACHE_OF_WORKER_MIN,
     CACHE_TIMEOUT_DEPRECATED,
     CACHE_TIMEOUT_OF_WORKER,
+    CACHE_ACQUIRE_OF_WORKER,
     RECOVERY_OPTS_OF_WORKER,
     CONNECT_TIMEOUT_OF_WORKER,
     PREPOST_TIMEOUT_OF_WORKER,
+    PING_TIMEOUT_OF_WORKER,
+    PING_MODE_OF_WORKER,
     REPLY_TIMEOUT_OF_WORKER,
     SOCKET_TIMEOUT_OF_WORKER,
+    SOCKET_CONNECT_TIMEOUT_OF_WORKER,
     SOCKET_BUFFER_OF_WORKER,
     SOCKET_KEEPALIVE_OF_WORKER,
+    CONN_PING_INTERVAL_OF_WORKER,
     RECYCLE_TIMEOUT_DEPRECATED,
     LOAD_FACTOR_OF_WORKER,
     DISTANCE_OF_WORKER,
@@ -263,6 +287,8 @@ static const char *supported_properties[] = {
     BALANCE_WORKERS,
     STICKY_SESSION,
     STICKY_SESSION_FORCE,
+    SESSION_COOKIE_OF_WORKER,
+    SESSION_PATH_OF_WORKER,
     LOCAL_WORKER_DEPRECATED,
     LOCAL_WORKER_ONLY_DEPRECATED,
     JVM_ROUTE_OF_WORKER_DEPRECATED,
@@ -276,7 +302,9 @@ static const char *supported_properties[] = {
     IS_WORKER_STOPPED_DEPRECATED,
     ACTIVATION_OF_WORKER,
     WORKER_RECOVER_TIME,
+    WORKER_ERROR_ESCALATION_TIME,
     MAX_REPLY_TIMEOUTS_OF_WORKER,
+    RETRY_INTERVAL_OF_WORKER,
     WORKER_MAX_PACKET_SIZE,
     STYLE_SHEET_OF_WORKER,
     NAMESPACE_OF_WORKER,
@@ -321,13 +349,13 @@ int jk_get_bool_code(const char *v, int def)
     else if (!strcasecmp(v, "off") ||
              *v == 'F' || *v == 'f' ||
              *v == 'N' || *v == 'n' ||
-             *v == '0') {
+            (*v == '0' && *(v + 1) == '\0')) {
         return 0;
     }
     else if (!strcasecmp(v, "on") ||
              *v == 'T' || *v == 't' ||
              *v == 'Y' || *v == 'y' ||
-             *v == '1') {
+            (*v == '1' && *(v + 1) == '\0')) {
         return 1;
     }
     return def;
@@ -346,8 +374,8 @@ void jk_sleep(int ms)
     Sleep(ms);
 #else
     struct timeval tv;
-    tv.tv_usec = ms * 1000;
-    tv.tv_sec = 0;
+    tv.tv_usec = (ms % 1000) * 1000;
+    tv.tv_sec = ms / 1000;
     select(0, NULL, NULL, NULL, &tv);
 #endif
 }
@@ -436,6 +464,9 @@ static int set_time_str(char *str, int len, jk_logger_t *l)
 {
     time_t t;
     struct tm *tms;
+#ifdef _MT_CODE_PTHREAD
+    struct tm res;
+#endif
     int done;
 /* We want to use a fixed maximum size buffer here.
  * If we would dynamically adjust it to the real format
@@ -488,7 +519,11 @@ static int set_time_str(char *str, int len, jk_logger_t *l)
 #else
     t = time(NULL);
 #endif
+#ifdef _MT_CODE_PTHREAD
+    tms = localtime_r(&t, &res);
+#else
     tms = localtime(&t);
+#endif
     if (log_fmt[0])
         done = (int)strftime(str, len, log_fmt, tms);
     else
@@ -577,6 +612,39 @@ int jk_open_file_logger(jk_logger_t **l, const char *file, int level)
     return JK_FALSE;
 }
 
+int jk_attach_file_logger(jk_logger_t **l, int fd, int level)
+{
+    if (l && fd >= 0) {
+
+        jk_logger_t *rc = (jk_logger_t *)malloc(sizeof(jk_logger_t));
+        jk_file_logger_t *p = (jk_file_logger_t *) malloc(sizeof(jk_file_logger_t));
+        if (rc && p) {
+            rc->log = log_to_file;
+            rc->level = level;
+            rc->logger_private = p;
+#if defined(AS400) && !defined(AS400_UTF8)
+            p->logfile = fdopen(fd, "a+, o_ccsid=0");
+#else
+            p->logfile = fdopen(fd, "a+");
+#endif
+            if (p->logfile) {
+                *l = rc;
+                jk_set_time_fmt(rc, NULL);
+                return JK_TRUE;
+            }
+        }
+        if (rc) {
+            free(rc);
+        }
+        if (p) {
+            free(p);
+        }
+
+        *l = NULL;
+    }
+    return JK_FALSE;
+}
+
 int jk_close_file_logger(jk_logger_t **l)
 {
     if (l && *l) {
@@ -748,7 +816,7 @@ const char *jk_get_worker_redirect(jk_map_t *m, const char *wname, const char *d
     if (!m || !wname) {
         return NULL;
     }
-   MAKE_WORKER_PARAM(REDIRECT_OF_WORKER);
+    MAKE_WORKER_PARAM(REDIRECT_OF_WORKER);
     return jk_map_get_string(m, buf, def);
 }
 
@@ -872,6 +940,18 @@ int jk_get_worker_cache_size_min(jk_map_t *m, const char *wname, int def)
     return jk_map_get_int(m, buf, def);
 }
 
+int jk_get_worker_cache_acquire_timeout(jk_map_t *m, const char *wname, int def)
+{
+    char buf[1024];
+
+    if (!m || !wname) {
+        return -1;
+    }
+
+    MAKE_WORKER_PARAM(CACHE_ACQUIRE_OF_WORKER);
+    return jk_map_get_int(m, buf, def);
+}
+
 int jk_get_worker_socket_timeout(jk_map_t *m, const char *wname, int def)
 {
     char buf[1024];
@@ -885,6 +965,19 @@ int jk_get_worker_socket_timeout(jk_map_t *m, const char *wname, int def)
     return jk_map_get_int(m, buf, def);
 }
 
+int jk_get_worker_socket_connect_timeout(jk_map_t *m, const char *wname, int def)
+{
+    char buf[1024];
+
+    if (!m || !wname) {
+        return -1;
+    }
+
+    MAKE_WORKER_PARAM(SOCKET_CONNECT_TIMEOUT_OF_WORKER);
+
+    return jk_map_get_int(m, buf, def);
+}
+
 int jk_get_worker_recover_timeout(jk_map_t *m, const char *wname, int def)
 {
     char buf[1024];
@@ -898,6 +991,19 @@ int jk_get_worker_recover_timeout(jk_map_t *m, const char *wname, int def)
     return jk_map_get_int(m, buf, def);
 }
 
+int jk_get_worker_error_escalation_time(jk_map_t *m, const char *wname, int def)
+{
+    char buf[1024];
+
+    if (!m || !wname) {
+        return -1;
+    }
+
+    MAKE_WORKER_PARAM(WORKER_ERROR_ESCALATION_TIME);
+
+    return jk_map_get_int(m, buf, def);
+}
+
 int jk_get_worker_max_reply_timeouts(jk_map_t *m, const char *wname, int def)
 {
     char buf[1024];
@@ -911,6 +1017,19 @@ int jk_get_worker_max_reply_timeouts(jk_map_t *m, const char *wname, int def)
     return jk_map_get_int(m, buf, def);
 }
 
+int jk_get_worker_retry_interval(jk_map_t *m, const char *wname, int def)
+{
+    char buf[1024];
+
+    if (!m || !wname) {
+        return -1;
+    }
+
+    MAKE_WORKER_PARAM(RETRY_INTERVAL_OF_WORKER);
+
+    return jk_map_get_int(m, buf, def);
+}
+
 int jk_get_worker_socket_buffer(jk_map_t *m, const char *wname, int def)
 {
     char buf[1024];
@@ -940,6 +1059,19 @@ int jk_get_worker_socket_keepalive(jk_map_t *m, const char *wname, int def)
     return jk_map_get_bool(m, buf, def);
 }
 
+int jk_get_worker_conn_ping_interval(jk_map_t *m, const char *wname, int def)
+{
+    char buf[1024];
+
+    if (!m || !wname) {
+        return -1;
+    }
+
+    MAKE_WORKER_PARAM(CONN_PING_INTERVAL_OF_WORKER);
+
+    return jk_map_get_int(m, buf, def);
+}
+
 int jk_get_worker_cache_timeout(jk_map_t *m, const char *wname, int def)
 {
     char buf[1024];
@@ -983,6 +1115,34 @@ int jk_get_worker_prepost_timeout(jk_map_t *m, const char *wname, int def)
     return jk_map_get_int(m, buf, def);
 }
 
+int jk_get_worker_ping_timeout(jk_map_t *m, const char *wname, int def)
+{
+    char buf[1024];
+
+    if (!m || !wname) {
+        return -1;
+    }
+
+    MAKE_WORKER_PARAM(PING_TIMEOUT_OF_WORKER);
+
+    return jk_map_get_int(m, buf, def);
+}
+
+int jk_get_worker_ping_mode(jk_map_t *m, const char *wname, int def)
+{
+    char buf[1024];
+    const char *v;
+
+    if (!m || !wname) {
+        return def;
+    }
+
+    MAKE_WORKER_PARAM(PING_MODE_OF_WORKER);
+
+    v = jk_map_get_string(m, buf, NULL);
+    return jk_ajp_get_cping_mode(v, def);
+}
+
 int jk_get_worker_reply_timeout(jk_map_t *m, const char *wname, int def)
 {
     char buf[1024];
@@ -1753,10 +1913,45 @@ int jk_get_worker_libpath(jk_map_t *m, const char *wname, const char **libpath)
     return JK_FALSE;
 }
 
+const char *jk_get_lb_session_cookie(jk_map_t *m, const char *wname, const char *def)
+{
+    char buf[1024];
+    if (!m || !wname) {
+        return NULL;
+    }
+    MAKE_WORKER_PARAM(SESSION_COOKIE_OF_WORKER);
+    return jk_map_get_string(m, buf, def);
+}
+
+const char *jk_get_lb_session_path(jk_map_t *m, const char *wname, const char *def)
+{
+    char buf[1024];
+    if (!m || !wname) {
+        return NULL;
+    }
+    MAKE_WORKER_PARAM(SESSION_PATH_OF_WORKER);
+    return jk_map_get_string(m, buf, def);
+}
+
+
+int is_http_status_fail(unsigned int http_status_fail_num,
+                        int *http_status_fail, int status)
+{
+    unsigned int i;
+    int soft_status = -1 * status;
+    for (i = 0; i < http_status_fail_num; i++) {
+        if (http_status_fail[i] == status)
+            return 1;
+        else if (http_status_fail[i] == soft_status)
+            return -1;
+    }
+    return 0;
+}
+
 char **jk_parse_sysprops(jk_pool_t *p, const char *sysprops)
 {
     char **rc = NULL;
-#ifdef _REENTRANT
+#ifdef _MT_CODE_PTHREAD
     char *lasts;
 #endif
 
@@ -1774,7 +1969,7 @@ char **jk_parse_sysprops(jk_pool_t *p, const char *sysprops)
             rc = jk_pool_alloc(p, (num_of_prps + 1) * sizeof(char *));
             if (rc) {
                 unsigned i = 0;
-#ifdef _REENTRANT
+#ifdef _MT_CODE_PTHREAD
                 char *tmp = strtok_r(prps, "*", &lasts);
 #else
                 char *tmp = strtok(prps, "*");
@@ -1782,7 +1977,7 @@ char **jk_parse_sysprops(jk_pool_t *p, const char *sysprops)
 
                 while (tmp && i < num_of_prps) {
                     rc[i] = tmp;
-#ifdef _REENTRANT
+#ifdef _MT_CODE_PTHREAD
                     tmp = strtok_r(NULL, "*", &lasts);
 #else
                     tmp = strtok(NULL, "*");
@@ -1847,6 +2042,7 @@ void jk_init_ws_service(jk_ws_service_t *s)
     s->ssl_cert_len = 0;
     s->ssl_cipher = NULL;
     s->ssl_session = NULL;
+    s->ssl_key_size = -1;
     s->headers_names = NULL;
     s->headers_values = NULL;
     s->num_headers = 0;
@@ -1854,14 +2050,63 @@ void jk_init_ws_service(jk_ws_service_t *s)
     s->attributes_values = NULL;
     s->num_attributes = 0;
     s->route = NULL;
-    s->retries = JK_RETRIES;
+    s->secret = NULL;
+    s->reco_buf = NULL;
+    s->reco_status = RECO_NONE;
+    s->flush_packets = JK_FALSE;
+    s->flush_header = JK_FALSE;
+    s->extension.reply_timeout = -1;
+    s->extension.use_server_error_pages = 0;
+    s->extension.activation = NULL;
+    s->extension.fail_on_status_size = 0;
+    s->extension.fail_on_status = NULL;
     s->response_started = JK_FALSE;
+    s->response_blocked = JK_FALSE;
+    s->http_response_status = JK_HTTP_OK;
+    s->uw_map = NULL;
+    s->start_response = NULL;
+    s->read = NULL;
+    s->write = NULL;
+    s->flush = NULL;
+    s->done = NULL;
+    s->disable_reuse = JK_FALSE;
     s->add_log_items = NULL;
     s->next_vhost = NULL;
     s->vhost_to_text = NULL;
     s->vhost_to_uw_map = NULL;
 }
 
+/* Match = 0, NoMatch = 1, Abort = -1
+ * Based loosely on sections of wildmat.c by Rich Salz
+ */
+int jk_wildchar_match(const char *str, const char *exp, int icase)
+{
+    int x, y;
+
+    for (x = 0, y = 0; exp[y]; ++y, ++x) {
+        if (!str[x] && exp[y] != '*')
+            return -1;
+        if (exp[y] == '*') {
+            while (exp[++y] == '*');
+            if (!exp[y])
+                return 0;
+            while (str[x]) {
+                int ret;
+                if ((ret = jk_wildchar_match(&str[x++], &exp[y], icase)) != 1)
+                    return ret;
+            }
+            return -1;
+        }
+        else if (exp[y] != '?') {
+            if (icase && (tolower(str[x]) != tolower(exp[y])))
+                return 1;
+            else if (!icase && str[x] != exp[y])
+                return 1;
+        }
+    }
+    return (str[x] != '\0');
+}
+
 #ifdef _MT_CODE_PTHREAD
 jk_uint32_t jk_gettid()
 {
