@@ -17,7 +17,7 @@
 
 /***************************************************************************
  * Description: JK version header file                                     *
- * Version:     $Revision: 606224 $                                           *
+ * Version:     $Revision: 757055 $                                           *
  ***************************************************************************/
 
 #ifndef __JK_VERSION_H
@@ -26,43 +26,47 @@
 /************** START OF AREA TO MODIFY BEFORE RELEASING *************/
 #define JK_VERMAJOR     1
 #define JK_VERMINOR     2
-#define JK_VERFIX       26
-#define JK_VERSTRING    "1.2.26"
+#define JK_VERFIX       28
+#define JK_VERSTRING    "1.2.28"
 
+/* set JK_VERISRELEASE to 1 when release (do not forget to commit!) */
+#define JK_VERISRELEASE 1
 /* Beta number */
 #define JK_VERBETA      0
 #define JK_BETASTRING   "0"
-/* set JK_VERISRELEASE to 1 when release (do not forget to commit!) */
-#define JK_VERISRELEASE 1
+/* Release candidate */
 #define JK_VERRC        0
 #define JK_RCSTRING     "0"
+/* Source Control Revision as a suffix, e.g. "-r12345" */
+#define JK_REVISION "-"
 
 /************** END OF AREA TO MODIFY BEFORE RELEASING *************/
 
 #if !defined(PACKAGE)
+#if defined(JK_ISAPI)
+#define PACKAGE "isapi_redirector"
+#elif defined(JK_NSAPI)
+#define PACKAGE "nsapi_redirector"
+#else
 #define PACKAGE "mod_jk"
 #endif
+#endif
 
 /* Build JK_EXPOSED_VERSION and JK_VERSION */
 #define JK_EXPOSED_VERSION_INT PACKAGE "/" JK_VERSTRING
 
-#if ( JK_VERISRELEASE == 1 )
-#define JK_RELEASE_STR  JK_EXPOSED_VERSION_INT
+#if (JK_VERBETA != 0)
+#define JK_EXPOSED_VERSION JK_EXPOSED_VERSION_INT "-beta-" JK_BETASTRING
 #else
-#define JK_RELEASE_STR  JK_EXPOSED_VERSION_INT "-dev"
-#endif
-
-#if ( JK_VERBETA == 0 )
-#define JK_EXPOSED_VERSION JK_RELEASE_STR
 #undef JK_VERBETA
 #define JK_VERBETA 255
-#else
-#define JK_EXPOSED_VERSION JK_RELEASE_STR "-beta-" JK_BETASTRING
-#endif
-
 #if (JK_VERRC != 0)
-#undef JK_EXPOSED_VERSION
-#define JK_EXPOSED_VERSION JK_RELEASE_STR "-rc-" JK_RCSTRING
+#define JK_EXPOSED_VERSION JK_EXPOSED_VERSION_INT "-rc-" JK_RCSTRING
+#elif (JK_VERISRELEASE == 1)
+#define JK_EXPOSED_VERSION JK_EXPOSED_VERSION_INT
+#else
+#define JK_EXPOSED_VERSION JK_EXPOSED_VERSION_INT "-dev" JK_REVISION
+#endif
 #endif
 
 #define JK_MAKEVERSION(major, minor, fix, beta) (((major) << 24) + ((minor) << 16) + ((fix) << 8) + (beta))
