@@ -19,7 +19,7 @@
  * Description: Workers controller                                         *
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
  * Author:      Henri Gomez <hgomez@apache.org>                            *
- * Version:     $Revision: 555180 $                                          *
+ * Version:     $Revision: 708019 $                                          *
  ***************************************************************************/
 
 #define _PLACE_WORKER_LIST_HERE
@@ -63,6 +63,7 @@ int wc_open(jk_map_t *init_data, jk_worker_env_t *we, jk_logger_t *l)
     }
 
     jk_map_dump(init_data, l);
+    we->init_data = init_data;
     if (!jk_get_worker_list(init_data, &(we->worker_list),
                             &we->num_of_workers)) {
         JK_TRACE_EXIT(l);
@@ -304,32 +305,47 @@ const char *wc_get_name_for_type(int type, jk_logger_t *l)
 void wc_maintain(jk_logger_t *l)
 {
     static time_t last_maintain = 0;
+    static int    running_maintain = 0;
     int sz = jk_map_size(worker_map);
 
     JK_TRACE_ENTER(l);
 
-    if (sz > 0 && worker_maintain_time > 0) {
+    /* Only proceed if all of the below hold true:
+     * - there are workers
+     * - maintenance wasn't disabled by configuration
+     * - time since last maintenance is big enough
+     */
+    if (sz > 0 && worker_maintain_time > 0 &&
+        difftime(time(NULL), last_maintain) >= worker_maintain_time) {
         int i;
-        time_t now;
         JK_ENTER_CS(&worker_lock, i);
-        now = time(NULL);
-        if (difftime(now, last_maintain) >= worker_maintain_time) {
-            last_maintain = now;
+        if (running_maintain ||
+            difftime(time(NULL), last_maintain) < worker_maintain_time) {
+            /* Already in maintain */
             JK_LEAVE_CS(&worker_lock, i);
-            for (i = 0; i < sz; i++) {
-                jk_worker_t *w = jk_map_value_at(worker_map, i);
-                if (w && w->maintain) {
-                    if (JK_IS_DEBUG_LEVEL(l))
-                        jk_log(l, JK_LOG_DEBUG,
-                               "Maintaining worker %s",
-                               jk_map_name_at(worker_map, i));
-                    w->maintain(w, now, l);
-                }
+            JK_TRACE_EXIT(l);
+            return;
+        }
+        /* Set the maintain run flag so other threads skip
+         * the maintain until we are finished.
+         */
+        running_maintain = 1;
+        JK_LEAVE_CS(&worker_lock, i);
+
+        for (i = 0; i < sz; i++) {
+            jk_worker_t *w = jk_map_value_at(worker_map, i);
+            if (w && w->maintain) {
+                if (JK_IS_DEBUG_LEVEL(l))
+                    jk_log(l, JK_LOG_DEBUG,
+                           "Maintaining worker %s",
+                           jk_map_name_at(worker_map, i));
+                w->maintain(w, time(NULL), l);
             }
         }
-        else {
-            JK_LEAVE_CS(&worker_lock, i);
-        }
+        JK_ENTER_CS(&worker_lock, i);
+        last_maintain = time(NULL);
+        running_maintain = 0;
+        JK_LEAVE_CS(&worker_lock, i);
     }
     JK_TRACE_EXIT(l);
 }
